@@ -1,5 +1,5 @@
 /* This file is part of the KDE Project
-   Copyright (c) 2007-2010 Sebastian Trueg <trueg@kde.org>
+   Copyright (c) 2007-2011 Sebastian Trueg <trueg@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -24,6 +24,7 @@
 #include "removabledeviceindexnotification.h"
 #include "removablemediacache.h"
 #include "../strigi/strigiserviceconfig.h"
+#include "activefilequeue.h"
 
 #ifdef BUILD_KINOTIFY
 #include "kinotify.h"
@@ -96,12 +97,12 @@ namespace {
 
         //Only watch the strigi index folders for file creation and change.
         if( Nepomuk::StrigiServiceConfig::self()->shouldFolderBeIndexed( path ) ) {
+            modes |= KInotify::EventCloseWrite;
             modes |= KInotify::EventCreate;
-            modes |= KInotify::EventModify;
         }
         else {
+            modes &= (~KInotify::EventCloseWrite);
             modes &= (~KInotify::EventCreate);
-            modes &= (~KInotify::EventModify);
         }
 
         return true;
@@ -113,6 +114,10 @@ namespace {
 Nepomuk::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
     : Service( parent )
 {
+    // Create the configuration instance singleton (for thread-safety)
+    // ==============================================================
+    (void)new StrigiServiceConfig(this);
+
     // the list of default exclude filters we use here differs from those
     // that can be configured for the strigi service
     // the default list should only contain files and folders that users are
@@ -123,11 +128,17 @@ Nepomuk::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
     m_pathExcludeRegExpCache->rebuildCacheFromFilterList( defaultExcludeFilterList() );
 
     // start the mover thread
-    m_metadataMover = new MetadataMover( mainModel(), this );
+    m_metadataMoverThread = new QThread(this);
+    m_metadataMoverThread->start();
+    m_metadataMover = new MetadataMover( mainModel() );
     connect( m_metadataMover, SIGNAL(movedWithoutData(QString)),
              this, SLOT(slotMovedWithoutData(QString)),
              Qt::QueuedConnection );
-    m_metadataMover->start();
+    m_metadataMover->moveToThread(m_metadataMoverThread);
+
+    m_fileModificationQueue = new ActiveFileQueue(this);
+    connect(m_fileModificationQueue, SIGNAL(urlTimeout(KUrl)),
+            this, SLOT(slotActiveFileQueueTimeout(KUrl)));
 
 #ifdef BUILD_KINOTIFY
     // monitor the file system for changes (restricted by the inotify limit)
@@ -137,10 +148,10 @@ Nepomuk::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
              this, SLOT( slotFileMoved( QString, QString ) ) );
     connect( m_dirWatch, SIGNAL( deleted( QString, bool ) ),
              this, SLOT( slotFileDeleted( QString, bool ) ) );
-    connect( m_dirWatch, SIGNAL( created( QString ) ),
-             this, SLOT( slotFileCreated( QString ) ) );
-    connect( m_dirWatch, SIGNAL( modified( QString ) ),
-             this, SLOT( slotFileModified( QString ) ) );
+    connect( m_dirWatch, SIGNAL( created( QString, bool ) ),
+             this, SLOT( slotFileCreated( QString, bool ) ) );
+    connect( m_dirWatch, SIGNAL( closedWrite( QString ) ),
+             this, SLOT( slotFileClosedAfterWrite( QString ) ) );
     connect( m_dirWatch, SIGNAL( watchUserLimitReached() ),
              this, SLOT( slotInotifyWatchUserLimitReached() ) );
 
@@ -174,18 +185,19 @@ Nepomuk::FileWatch::FileWatch( QObject* parent, const QList<QVariant>& )
 Nepomuk::FileWatch::~FileWatch()
 {
     kDebug();
-    m_metadataMover->stop();
-    m_metadataMover->wait();
+    m_metadataMoverThread->quit();
+    m_metadataMoverThread->wait();
 }
 
 
+// FIXME: listen to Create for folders!
 void Nepomuk::FileWatch::watchFolder( const QString& path )
 {
     kDebug() << path;
 #ifdef BUILD_KINOTIFY
     if ( m_dirWatch && !m_dirWatch->watchingPath( path ) )
         m_dirWatch->addWatch( path,
-                              KInotify::WatchEvents( KInotify::EventMove|KInotify::EventDelete|KInotify::EventDeleteSelf|KInotify::EventCreate|KInotify::EventModify ),
+                              KInotify::WatchEvents( KInotify::EventMove|KInotify::EventDelete|KInotify::EventDeleteSelf|KInotify::EventCloseWrite|KInotify::EventCreate ),
                               KInotify::WatchFlags() );
 #endif
 }
@@ -193,12 +205,9 @@ void Nepomuk::FileWatch::watchFolder( const QString& path )
 
 void Nepomuk::FileWatch::slotFileMoved( const QString& urlFrom, const QString& urlTo )
 {
-    if( !ignorePath( urlFrom ) ) {
-        KUrl from( urlFrom );
-        KUrl to( urlTo );
-
-        kDebug() << from << to;
-
+    if( !ignorePath( urlFrom ) || !ignorePath( urlTo ) ) {
+        const KUrl from( urlFrom );
+        const KUrl to( urlTo );
         m_metadataMover->moveFileMetadata( from, to );
     }
 }
@@ -231,22 +240,27 @@ void Nepomuk::FileWatch::slotFileDeleted( const QString& urlString, bool isDir )
 }
 
 
-void Nepomuk::FileWatch::slotFileCreated( const QString& path )
+void Nepomuk::FileWatch::slotFileCreated( const QString& path, bool isDir )
 {
-    kDebug() << path;
-    updateFileViaStrigi( path );
+    // we only need the file creation event for folders
+    // file creation is always followed by a CloseAfterWrite event
+    if(isDir) {
+        updateFileViaStrigi(path);
+    }
 }
 
 
-void Nepomuk::FileWatch::slotFileModified( const QString& path )
+void Nepomuk::FileWatch::slotFileClosedAfterWrite( const QString& path )
 {
-    updateFileViaStrigi( path );
+    if(StrigiServiceConfig::self()->shouldBeIndexed(path)) {
+        // we do not tell the file indexer right away but wait a short while in case the file is modified very often (irc logs for example)
+        m_fileModificationQueue->enqueueUrl( path );
+    }
 }
 
-
 void Nepomuk::FileWatch::slotMovedWithoutData( const QString& path )
 {
-    updateFolderViaStrigi( path );
+    updateFileViaStrigi( path );
 }
 
 
@@ -378,4 +392,9 @@ void Nepomuk::FileWatch::slotDeviceMounted(const Nepomuk::RemovableMediaCache::E
     watchFolder(entry->mountPath());
 }
 
+void Nepomuk::FileWatch::slotActiveFileQueueTimeout(const KUrl &url)
+{
+    updateFileViaStrigi(url.toLocalFile());
+}
+
 #include "nepomukfilewatch.moc"
