@@ -62,6 +62,9 @@ Nepomuk::Repository::Repository( const QString& name )
       m_model( 0 ),
       m_inferencer( 0 ),
       m_removableStorageModel( 0 ),
+      m_dataManagementModel( 0 ),
+      m_dataManagementAdaptor( 0 ),
+      m_nrlModel( 0 ),
       m_backend( 0 ),
       m_modelCopyJob( 0 ),
       m_oldStorageBackend( 0 )
@@ -86,9 +89,13 @@ void Nepomuk::Repository::close()
     delete m_dataManagementAdaptor;
     m_dataManagementAdaptor = 0;
 
+    setParentModel(0);
     delete m_dataManagementModel;
     m_dataManagementModel = 0;
 
+    delete m_classAndPropertyTree;
+    m_classAndPropertyTree = 0;
+
     delete m_inferencer;
     m_inferencer = 0;
 
@@ -185,6 +192,10 @@ void Nepomuk::Repository::open()
         return;
     }
 
+#if SOPRANO_IS_VERSION(2, 7, 3)
+    connect(m_model, SIGNAL(virtuosoStopped(bool)), this, SLOT(slotVirtuosoStopped(bool)));
+#endif
+
     kDebug() << "Successfully created new model for repository" << name();
 
     // Fire up the graph maintainer on the pure data model.
@@ -208,12 +219,12 @@ void Nepomuk::Repository::open()
     // create a SignalCacheModel to make sure no client slows us down by listening to the stupid signals
     // =================================
     Soprano::Util::SignalCacheModel* scm = new Soprano::Util::SignalCacheModel( m_removableStorageModel );
-    scm->setParent(this); // memory management
+    scm->setParent(m_removableStorageModel); // memory management
 
     // Create the NRLModel which is required by the DMM below
     // =================================
     m_nrlModel = new Soprano::NRLModel(scm);
-    m_nrlModel->setParent(this); // memory management
+    m_nrlModel->setParent(scm); // memory management
 
     // create the DataManagementModel on top of everything
     // =================================
@@ -420,4 +431,15 @@ void Nepomuk::Repository::updateInference()
     m_inferencer->updateAllResources();
 }
 
+void Nepomuk::Repository::slotVirtuosoStopped(bool normalExit)
+{
+    if(!normalExit) {
+        kDebug() << "Virtuoso was killed or crashed. Restarting the repository.";
+        // restart the dumb way for now
+        // Ideally we would inform the other services so they can be restarted or something.
+        close();
+        open();
+    }
+}
+
 #include "repository.moc"
